@@ -4,7 +4,7 @@ use warnings;
 
 package Art::World {
 
-  our $VERSION = '0.18_17';
+  our $VERSION = '0.18_18';
 
   use Zydeco
     authority => 'cpan:SMONFF',
@@ -55,39 +55,85 @@ package Art::World {
   }
 
   role Buyer {
+
     requires money;
 
-    method $change_owner ( Agent *new_owner, Artwork *artwork ) {
-      $arg->artwork->owner->@* = ();
-      push $arg->artwork->owner->@*, $arg->new_owner ;
-      push $arg->new_owner->collection->@*, $arg->artwork;
-      # TODO guess it should bump some people reputation and artwork aura now
+    multi method acquire ( Artwork $art ) {
+      $self->pay( $art, $self );
+      $art->change_owner( $self );
     }
+  }
 
-    # TODO test the case when there are many buyers/owners
-    method acquire ( Artwork $art ) {
-      $self->pay( $art );
-      $art->$change_owner({ new_owner => $self, artwork => $art });
+  role Collective {
+    has members! ( type => ArrayRef[ Object ] );
+    multi method acquire ( Artwork *art, Collective *collective ) {
+      for my $member ( $arg->collective->members->@* ) {
+        $member->pay( $arg->art, $arg->collective );
+      }
+      $arg->art->change_owner( $arg->collective );
     }
   }
 
   # Dunno how but the fact of being collected should bump artist reputation
   role Collectionable {
-    has owner  (
-      default => sub { $_[0]->creator },
-      is => rw,
-      type => ArrayRef[ Agent ]);
+    has owner (
+      lazy    => true,
+      is      => rw,
+      clearer => true,
+      writer  => 'set_owner',
+      type    => ArrayRef[ Agent ]) = $self->creator;
     has value  ( is   => rw );
     has status (
       enum            => ['for_sale', 'sold'],
       # Create a delegated method for each value in the enumeration
       handles         => 1,
       default         => 'for_sale' );
+
+    method $remove_from_seller_collection {
+      my $meta = Art::World::Util->new_meta;
+      # Removal of the to-be-sold Artwork in seller collection
+      for my $owner ( $self->owner->@* ) {
+        # Artists don't have a collection
+        # Or maybe they can... So we should add a specific case
+        if (  $meta->get_class( $owner ) !~ '^Art::World::Artist$' ) {
+          while ( my ( $i, $art ) = each( $owner->collection->@* )) {
+            if ( $art->title =~ $self->title ) {
+              # Wish I would use List::UtilsBy extract_by() for this
+              splice $owner->collection->@*, $i, 1;
+            }
+          }
+        }
+      }
+    }
+
+    multi method change_owner ( Collector $buyer ) {
+
+      # From seller collection
+      $self->$remove_from_seller_collection;
+
+      $self->clear_owner;
+      $self->set_owner([ $buyer ]);
+      push $buyer->collection->@*, $self;
+      # TODO guess it should bump some people reputation and artwork aura now
+    }
+
+    multi method change_owner ( Coinvestor $buyers ) {
+
+      # From Collector point of view
+      $self->$remove_from_seller_collection;
+
+      # From Artwork point of view
+      $self->clear_owner;
+      $self->set_owner( $buyers->members );
+      push $buyers->collection->@*, $self;
+      # TODO guess it should bump some people reputation and artwork aura now
+    }
+
   }
 
   role Crud {
     has dbh! ( type => InstanceOf[ 'Art::World::Model' ], builder => true, lazy => true );
-    has db! ( type => HashRef[ Str ], builder => true, lazy => true );
+    has db!  ( type => HashRef[ Str ], builder => true, lazy => true );
 
     method insert ( Str $table, HashRef $attributes ) {
       # TODO IT MUST BE TESTED
@@ -213,12 +259,24 @@ package Art::World {
   }
 
   role Market {
+
     has money! ( type => Num, is => rw, default => 0 );
-    method pay ( Artwork $piece  ) {
+
+    # Can be a personal collector or a Coinvestor
+    method pay ( Artwork $piece, Collector $collector  ) {
+      # Divide what must be paid by each buyer
+      my $must_give = Art::World::Util
+        ->new_meta
+        ->get_class( $collector ) !~ /^Art::World::Coinvestor$/ ?
+          $piece->value :
+          $piece->value / scalar $collector->members->@*;
+
       # The money must be divided in equal between all owners
-      my $part = $piece->value / scalar $piece->owner->@*;
-      map { $_->money( $_->money - $part ) } $piece->owner->@*;
-      $self->money( $self->money - $piece->value );
+      my $part =  $must_give / scalar $piece->owner->@*;
+      # Seller(s) got their money given
+      map { $_->money( $_->money + $part ) } $piece->owner->@*;
+      # Buyer gives the money
+      $self->money( $self->money - $must_give );
     }
   }
 
@@ -231,7 +289,7 @@ package Art::World {
 
   role Space {
     # Like a small space or a large space
-    # Could limit the number of person coming
+    # Could limit the number of person coming during an event for example
     has space;
   }
 
@@ -275,8 +333,6 @@ package Art::World {
 
     class Playground with Crud, Identity {
 
-      class Collective;
-
       class Magazine {
 
         has reader;
@@ -301,8 +357,10 @@ package Art::World {
           class Museum with Exhibit, Buyer;
 
           class School {
-            has student;
-            has teachers;
+            # TODO much underground
+            has student  ( type => ArrayRef[ Agent ]);
+            # TODO Should enforce a minimum reputation
+            has teachers ( type => ArrayRef[ Agent ]);;
           }
         }
 
@@ -350,7 +408,7 @@ package Art::World {
       class Artist with Market {
 
         has artworks   ( type => ArrayRef );
-        has collectors ( type => ArrayRef[ Collector ], default => sub { [] } );
+        has collectors ( type => ArrayRef[ Collector ], default => sub { [] });
         has collected  ( type => Bool, default => false, is => rw );
         has status (
           enum => [ 'underground', 'homogenic' ],
@@ -379,12 +437,14 @@ package Art::World {
         }
       }
 
-
       class Collector with Active, Buyer, Market {
         has collection (
           type    => ArrayRef[ Artwork, 0 ],
           default => sub { [] },
           is      => rw, );
+        class Coinvestor with Collective {
+          # TODO the Coinvestor money should be automatically build from all the investors
+        }
       }
 
       class Critic with Language, Writing {};
@@ -415,10 +475,9 @@ package Art::World {
     class Work extends Concept with Fame {
 
       has creation_date;
-      has creator(
+      has creator!(
         is   => ro,
-        # TODO Should be an ArrayRef of Agents
-        type => ArrayRef[ Object ] );
+        type => ArrayRef[ Agent ] );
 
       has title(
         is   => ro,
@@ -462,8 +521,9 @@ Art::World - Modeling of creative processes
 
   my $artwork = Art::World->new_artwork(
     creator => [ $artist, $another_artist ]  ,
-    value   => 100,
-    owner   => 'smonff' );
+    owner   => 'smonff'
+    title   => 'Corrupted art institutions likes critical art'
+    value   => 100, );
 
   my $museum = Art::World->new_museum(
     money => 1000,
@@ -498,6 +558,9 @@ Zydeco toolkit by Toby Inkster and immediatly felt very enthusiastic to use it
 to implement my idea. I hope it is a decent example of the possibilities this
 wonderful framework make possible.
 
+It is possible that this toolkit may be used by an art management software as it
+could be needed in an art galery or a museum.
+
 =head1 ROLES
 
 =head2 Abstraction
@@ -518,15 +581,21 @@ there is no such thing as somebody in the art world that buy but doesn't sale.
 The C<aquire> method is requiring some C<money>. The C<money> is provided by the
 C<Market> role.
 
-  $collector->acquire( $artwork );
+  $collector->acquire({ art => $artwork });
+
+Those behaviors are delegated to the private C<$change_owner()> method, and to
+the C<pay()> method from the C<Market> role.
 
 When a C<Collector> C<acquire()> an C<Artwork>, the C<Artwork> C<ArrayRef> of
 owners is automatically cleared and substituted by the new owner(s).
 C<acquire()> will remove the value of the C<Artwork> from the owner's money.
 
-Those behaviors are delegated to the private C<$change_owner()> method, and to
-the C<pay()> method from the C<Market> role.
+  $artwork->change_owner({ buyer => $sef });
 
+When the paiement occurs, the money is allocated to all the artwork owners
+involved and removed from the buyer's fortune.
+
+  $self->pay( $artwork );
 
 =head2 Collectionable
 
@@ -535,6 +604,32 @@ The C<collectionnable> provide a series of attributes.
 If it's collectionable, it can go to a C<Collector> collection or in a
 C<Museum>. A collectionable item is automatically owned by it's creator.
 
+=head2 Collective
+
+They do stuff together. You know, art is not about lonely C<Artists> in their
+C<Workshop>.
+
+This has first been designed for collectors, but could be used to
+activate any kind of collective behavior.
+
+This is quite a problem because what if a group of people wants to buy? We have
+a C<Coinvestor> class implementing the C<Collective> role that provide a
+I<collective-acquirement> method.
+
+It's C<acquire()> multi method provide a way to collectively buy an item.
+C<Coinvestor> is a class inheriting from C<Collector> that implement the
+C<Collective> role so it would be able to collectively acquire C<Works>.
+
+Note that the signatures are different between C<Buyer> and C<Collective> roles:
+
+  $collector->acquire({ art => $artwork });
+    # ==> using the method from the Buyer role
+
+  $coinvestors->acquire({ art => $collector->collection->[2], collective => $coinvestors });
+    # ==> using the method from the Collective role
+
+Just pass a self-reference to the coinvestor object and they will be able to organize and buy together.
+
 =head2 Crud
 
 The C<Crud> role makes possible to serialize most of the entities. For this to be
@@ -625,6 +720,10 @@ It is all about offer and demand. Involve C<money>. It provide a C<pay()> method
 that can be using by entities consumming this role. This method accept an
 C<Artwork> as an unique required parameter.
 
+The C<pay()> method is used by an entity that is designed by C<$self>.
+
+TODO How many people can buy a thing together?
+
 =head2 Manager
 
 A role for those who I<take care> of exhibitions and other organizational
@@ -721,10 +820,6 @@ Where a lot of theory is written by C<Critics>
 
 =head2 Collector
 
-=head2 Collective
-
-They do stuff together. You know, art is not about lonely C<Artists> in their C<Workshop>.
-
 =head2 Concept
 
 C<Concept> is an abstract class that does the C<Abstraction> role.
